package edi.upvehu.gbg.tap.demos;

import java.io.IOException;
import javax.swing.JOptionPane;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.HashSet;

public class DBClase {
    //URL base donde se encuentran los ficheros a procesar.
    private static final String URLDIR="http://gtts.ehu.es/German/Docencia/assets/";
    //Nombres de los ficheros a procesar.
    private static final String[] FILENAMES={"20171201.txt", "20171211.txt", "20171214.txt", "20171215.txt"};

    public static void main(String[] args) {

            // Recorremos todos los ficheros uno a uno
            for (String filename : FILENAMES) {
                try {
                    // Se procesa el fichero obteniendo un objeto que es un conjunto de objetos de nuestro interés
                    FirstPhoneneSet f=new FirstPhoneneSet(URLDIR+filename);
                    // y con él se cumplimentan los cinco campos de statement que tenemos preparado
                    System.out.format("Fichero %s: %d palabras (válidas: %d) [media=%.4f, desviación=%.4f]\n",
                            filename, f.palabrasTotales(), f.palabrasValidas(), f.media(), f.desviacion());
                } catch (IOException ex) {
                    //En caso de problemas se muestra un mensaje con un cuadro de dialogo.
                    //no es la mejor opción porque paraliza la ejecución hasta que se atiende.
                    //TODO estudiar lo que hace la linea que pone Netbeans "de oficio" (que es la solución correcta)
                    JOptionPane.showMessageDialog(null,"<html>Lo sentimos:<br/>"+ex.getMessage());
                }
            }
    }
}


/**  ================================== CLASE FirstPhoneneSet  ========================================
 * HashSet que almacena objetos de la clase interna FirstPhoneme.
 * Se construye a partir de un fichero con formato de alineamiento fonético.
 * que representan los fonemas iniciales de palabra.
 * Los fonemas almacenados son los iniciales de toda palabra que tiene asignado un fonema con su duración.
 * Las palabras que no tienen fonema y duración son contadas como palabras no válidas
 */
 class FirstPhoneneSet extends HashSet<FirstPhoneneSet.FirstPhoneme>{

    /** Clase que representa un fonema inicial de palabra por la palabra y su duración.
     */
    public static class FirstPhoneme{
        private final String palabra;
        private final double duracion;

        public FirstPhoneme(String palabra, double duracion) {
            this.palabra = palabra;
            this.duracion = duracion;
        }
    }

    // Contador de palabras inválidas
    private int palabrasInvalidas=0;

    /**
     * Constructor a partir de un fichero con formato de alineamiento fonético.
     * @param fileUrl la URL del fichero.
     * @throws IOException no es posible leer el fichero o el parámetro fileUrl no es una URL está bien formada.
     */
    public FirstPhoneneSet(String fileUrl) throws IOException {
        // Se genera un objeto URL con la string, se le pide un stream de datos y se envuelve convenientemente.
        BufferedReader br=new BufferedReader(new InputStreamReader(new URL(fileUrl).openStream()));
        // Leemos el fichero linea a linea
        String linea;
        while ( (linea=br.readLine()) !=null) {
            // Rechazamos todas las lineas que con coincidan con inicio de palabra.
            if (!linea.matches(".*\\|[^\\|]+\\|.*\\|.*")) continue;
            // Si no tienen valores de tiempo es palabra inválida
            if (linea.matches(".*\\|[^\\|]+\\|\\|.*")) {palabrasInvalidas++;continue;}
            // Obtenemos los tres valores que nos interesan en un array de String 
            String[] arr=linea.replaceFirst(".*\\|([^\\|]+)\\|(.+) (.+) .*\\|.*", "$1 $2 $3").split(" ");
            // y almacenamos un objeto FirstPhoneme con la palabra y la duración (diferencia de tiempos)
            add(new FirstPhoneme(arr[0],Double.parseDouble(arr[2])-Double.parseDouble(arr[1])));
           }
    }

    /**
     * Cálculo de la media de las duraciones de todos los elementos del conjunto.
     * @return el valor de la media.
     */
    public double media(){
        double sumatorio=0.0;
        for (FirstPhoneme p:this) sumatorio+=p.duracion;
        return sumatorio/size();
    }

    /**
     * Cáculo de la desviación típica de las duraciones de todos los elementos del conjunto.
     * @return el valor de la desviación típica.
     */
    public double desviacion(){
        double sumatorio=0.0, media=media();
        for (FirstPhoneme p:this) sumatorio+=(p.duracion-media)*(p.duracion-media);
        return Math.sqrt(sumatorio/(size()-1));
    }

    /**
     * Número de palabras en el conjunto (las palabras válidas)
     * @return el número de palabras válidas.
     */
    public int palabrasValidas(){ return size(); }

    /**
     * Número de palabras totales en el fichero procesado (suma de válidas e inválidas)
     * @return el número de palabras totales en el fichero procesado.
     */
    public int palabrasTotales(){ return palabrasValidas() + palabrasInvalidas; }
}
